# coding: utf-8

"""
    (v2.0) Dynamics 365 Business Central

    (v2.0) Business Central Standard APIs

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from pybusinesscentral.model.currency import Currency
from pybusinesscentral.model.customer import Customer
from pybusinesscentral.model.payment_term import PaymentTerm
from pybusinesscentral.model.postaladdresstype import Postaladdresstype
from pybusinesscentral.model.sales_order_line import SalesOrderLine
from pybusinesscentral.model.shipment_method import ShipmentMethod
from typing import Optional, Set
from typing_extensions import Self

class SalesOrder(BaseModel):
    """
    SalesOrder
    """ # noqa: E501
    id: Optional[StrictStr] = Field(default=None, description="(v1.0) The id property for the Dynamics 365 Business Central salesOrder entity")
    number: Optional[Annotated[str, Field(strict=True, max_length=20)]] = Field(default=None, description="(v1.0) The number property for the Dynamics 365 Business Central salesOrder entity")
    external_document_number: Optional[Annotated[str, Field(strict=True, max_length=35)]] = Field(default=None, description="(v1.0) The externalDocumentNumber property for the Dynamics 365 Business Central salesOrder entity", alias="externalDocumentNumber")
    order_date: Optional[datetime] = Field(default=None, description="(v1.0) The orderDate property for the Dynamics 365 Business Central salesOrder entity", alias="orderDate")
    customer_id: Optional[StrictStr] = Field(default=None, description="(v1.0) The customerId property for the Dynamics 365 Business Central salesOrder entity", alias="customerId")
    contact_id: Optional[Annotated[str, Field(strict=True, max_length=250)]] = Field(default=None, description="(v1.0) The contactId property for the Dynamics 365 Business Central salesOrder entity", alias="contactId")
    customer_number: Optional[Annotated[str, Field(strict=True, max_length=20)]] = Field(default=None, description="(v1.0) The customerNumber property for the Dynamics 365 Business Central salesOrder entity", alias="customerNumber")
    customer_name: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="(v1.0) The customerName property for the Dynamics 365 Business Central salesOrder entity", alias="customerName")
    bill_to_name: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="(v1.0) The billToName property for the Dynamics 365 Business Central salesOrder entity", alias="billToName")
    bill_to_customer_id: Optional[StrictStr] = Field(default=None, description="(v1.0) The billToCustomerId property for the Dynamics 365 Business Central salesOrder entity", alias="billToCustomerId")
    bill_to_customer_number: Optional[Annotated[str, Field(strict=True, max_length=20)]] = Field(default=None, description="(v1.0) The billToCustomerNumber property for the Dynamics 365 Business Central salesOrder entity", alias="billToCustomerNumber")
    ship_to_name: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="(v1.0) The shipToName property for the Dynamics 365 Business Central salesOrder entity", alias="shipToName")
    ship_to_contact: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="(v1.0) The shipToContact property for the Dynamics 365 Business Central salesOrder entity", alias="shipToContact")
    selling_postal_address: Optional[Postaladdresstype] = Field(default=None, alias="sellingPostalAddress")
    billing_postal_address: Optional[Postaladdresstype] = Field(default=None, alias="billingPostalAddress")
    shipping_postal_address: Optional[Postaladdresstype] = Field(default=None, alias="shippingPostalAddress")
    currency_id: Optional[StrictStr] = Field(default=None, description="(v1.0) The currencyId property for the Dynamics 365 Business Central salesOrder entity", alias="currencyId")
    currency_code: Optional[StrictStr] = Field(default=None, description="(v1.0) The currencyCode property for the Dynamics 365 Business Central salesOrder entity", alias="currencyCode")
    prices_include_tax: Optional[StrictBool] = Field(default=None, description="(v1.0) The pricesIncludeTax property for the Dynamics 365 Business Central salesOrder entity", alias="pricesIncludeTax")
    payment_terms_id: Optional[StrictStr] = Field(default=None, description="(v1.0) The paymentTermsId property for the Dynamics 365 Business Central salesOrder entity", alias="paymentTermsId")
    shipment_method_id: Optional[StrictStr] = Field(default=None, description="(v1.0) The shipmentMethodId property for the Dynamics 365 Business Central salesOrder entity", alias="shipmentMethodId")
    salesperson: Optional[Annotated[str, Field(strict=True, max_length=20)]] = Field(default=None, description="(v1.0) The salesperson property for the Dynamics 365 Business Central salesOrder entity")
    partial_shipping: Optional[StrictBool] = Field(default=None, description="(v1.0) The partialShipping property for the Dynamics 365 Business Central salesOrder entity", alias="partialShipping")
    requested_delivery_date: Optional[datetime] = Field(default=None, description="(v1.0) The requestedDeliveryDate property for the Dynamics 365 Business Central salesOrder entity", alias="requestedDeliveryDate")
    discount_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="(v1.0) The discountAmount property for the Dynamics 365 Business Central salesOrder entity", alias="discountAmount")
    discount_applied_before_tax: Optional[StrictBool] = Field(default=None, description="(v1.0) The discountAppliedBeforeTax property for the Dynamics 365 Business Central salesOrder entity", alias="discountAppliedBeforeTax")
    total_amount_excluding_tax: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="(v1.0) The totalAmountExcludingTax property for the Dynamics 365 Business Central salesOrder entity", alias="totalAmountExcludingTax")
    total_tax_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="(v1.0) The totalTaxAmount property for the Dynamics 365 Business Central salesOrder entity", alias="totalTaxAmount")
    total_amount_including_tax: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="(v1.0) The totalAmountIncludingTax property for the Dynamics 365 Business Central salesOrder entity", alias="totalAmountIncludingTax")
    fully_shipped: Optional[StrictBool] = Field(default=None, description="(v1.0) The fullyShipped property for the Dynamics 365 Business Central salesOrder entity", alias="fullyShipped")
    status: Optional[StrictStr] = Field(default=None, description="(v1.0) The status property for the Dynamics 365 Business Central salesOrder entity")
    last_modified_date_time: Optional[datetime] = Field(default=None, description="(v1.0) The lastModifiedDateTime property for the Dynamics 365 Business Central salesOrder entity", alias="lastModifiedDateTime")
    phone_number: Optional[Annotated[str, Field(strict=True, max_length=30)]] = Field(default=None, description="(v1.0) The phoneNumber property for the Dynamics 365 Business Central salesOrder entity", alias="phoneNumber")
    email: Optional[Annotated[str, Field(strict=True, max_length=80)]] = Field(default=None, description="(v1.0) The email property for the Dynamics 365 Business Central salesOrder entity")
    sales_order_lines: Optional[List[SalesOrderLine]] = Field(default=None, alias="salesOrderLines")
    customer: Optional[Customer] = None
    currency: Optional[Currency] = None
    payment_term: Optional[PaymentTerm] = Field(default=None, alias="paymentTerm")
    shipment_method: Optional[ShipmentMethod] = Field(default=None, alias="shipmentMethod")
    __properties: ClassVar[List[str]] = ["id", "number", "externalDocumentNumber", "orderDate", "customerId", "contactId", "customerNumber", "customerName", "billToName", "billToCustomerId", "billToCustomerNumber", "shipToName", "shipToContact", "sellingPostalAddress", "billingPostalAddress", "shippingPostalAddress", "currencyId", "currencyCode", "pricesIncludeTax", "paymentTermsId", "shipmentMethodId", "salesperson", "partialShipping", "requestedDeliveryDate", "discountAmount", "discountAppliedBeforeTax", "totalAmountExcludingTax", "totalTaxAmount", "totalAmountIncludingTax", "fullyShipped", "status", "lastModifiedDateTime", "phoneNumber", "email", "salesOrderLines", "customer", "currency", "paymentTerm", "shipmentMethod"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SalesOrder from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of selling_postal_address
        if self.selling_postal_address:
            _dict['sellingPostalAddress'] = self.selling_postal_address.to_dict()
        # override the default output from pydantic by calling `to_dict()` of billing_postal_address
        if self.billing_postal_address:
            _dict['billingPostalAddress'] = self.billing_postal_address.to_dict()
        # override the default output from pydantic by calling `to_dict()` of shipping_postal_address
        if self.shipping_postal_address:
            _dict['shippingPostalAddress'] = self.shipping_postal_address.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in sales_order_lines (list)
        _items = []
        if self.sales_order_lines:
            for _item_sales_order_lines in self.sales_order_lines:
                if _item_sales_order_lines:
                    _items.append(_item_sales_order_lines.to_dict())
            _dict['salesOrderLines'] = _items
        # override the default output from pydantic by calling `to_dict()` of customer
        if self.customer:
            _dict['customer'] = self.customer.to_dict()
        # override the default output from pydantic by calling `to_dict()` of currency
        if self.currency:
            _dict['currency'] = self.currency.to_dict()
        # override the default output from pydantic by calling `to_dict()` of payment_term
        if self.payment_term:
            _dict['paymentTerm'] = self.payment_term.to_dict()
        # override the default output from pydantic by calling `to_dict()` of shipment_method
        if self.shipment_method:
            _dict['shipmentMethod'] = self.shipment_method.to_dict()
        # set to None if number (nullable) is None
        # and model_fields_set contains the field
        if self.number is None and "number" in self.model_fields_set:
            _dict['number'] = None

        # set to None if external_document_number (nullable) is None
        # and model_fields_set contains the field
        if self.external_document_number is None and "external_document_number" in self.model_fields_set:
            _dict['externalDocumentNumber'] = None

        # set to None if order_date (nullable) is None
        # and model_fields_set contains the field
        if self.order_date is None and "order_date" in self.model_fields_set:
            _dict['orderDate'] = None

        # set to None if customer_id (nullable) is None
        # and model_fields_set contains the field
        if self.customer_id is None and "customer_id" in self.model_fields_set:
            _dict['customerId'] = None

        # set to None if contact_id (nullable) is None
        # and model_fields_set contains the field
        if self.contact_id is None and "contact_id" in self.model_fields_set:
            _dict['contactId'] = None

        # set to None if customer_number (nullable) is None
        # and model_fields_set contains the field
        if self.customer_number is None and "customer_number" in self.model_fields_set:
            _dict['customerNumber'] = None

        # set to None if customer_name (nullable) is None
        # and model_fields_set contains the field
        if self.customer_name is None and "customer_name" in self.model_fields_set:
            _dict['customerName'] = None

        # set to None if bill_to_name (nullable) is None
        # and model_fields_set contains the field
        if self.bill_to_name is None and "bill_to_name" in self.model_fields_set:
            _dict['billToName'] = None

        # set to None if bill_to_customer_id (nullable) is None
        # and model_fields_set contains the field
        if self.bill_to_customer_id is None and "bill_to_customer_id" in self.model_fields_set:
            _dict['billToCustomerId'] = None

        # set to None if bill_to_customer_number (nullable) is None
        # and model_fields_set contains the field
        if self.bill_to_customer_number is None and "bill_to_customer_number" in self.model_fields_set:
            _dict['billToCustomerNumber'] = None

        # set to None if ship_to_name (nullable) is None
        # and model_fields_set contains the field
        if self.ship_to_name is None and "ship_to_name" in self.model_fields_set:
            _dict['shipToName'] = None

        # set to None if ship_to_contact (nullable) is None
        # and model_fields_set contains the field
        if self.ship_to_contact is None and "ship_to_contact" in self.model_fields_set:
            _dict['shipToContact'] = None

        # set to None if selling_postal_address (nullable) is None
        # and model_fields_set contains the field
        if self.selling_postal_address is None and "selling_postal_address" in self.model_fields_set:
            _dict['sellingPostalAddress'] = None

        # set to None if billing_postal_address (nullable) is None
        # and model_fields_set contains the field
        if self.billing_postal_address is None and "billing_postal_address" in self.model_fields_set:
            _dict['billingPostalAddress'] = None

        # set to None if shipping_postal_address (nullable) is None
        # and model_fields_set contains the field
        if self.shipping_postal_address is None and "shipping_postal_address" in self.model_fields_set:
            _dict['shippingPostalAddress'] = None

        # set to None if currency_id (nullable) is None
        # and model_fields_set contains the field
        if self.currency_id is None and "currency_id" in self.model_fields_set:
            _dict['currencyId'] = None

        # set to None if currency_code (nullable) is None
        # and model_fields_set contains the field
        if self.currency_code is None and "currency_code" in self.model_fields_set:
            _dict['currencyCode'] = None

        # set to None if prices_include_tax (nullable) is None
        # and model_fields_set contains the field
        if self.prices_include_tax is None and "prices_include_tax" in self.model_fields_set:
            _dict['pricesIncludeTax'] = None

        # set to None if payment_terms_id (nullable) is None
        # and model_fields_set contains the field
        if self.payment_terms_id is None and "payment_terms_id" in self.model_fields_set:
            _dict['paymentTermsId'] = None

        # set to None if shipment_method_id (nullable) is None
        # and model_fields_set contains the field
        if self.shipment_method_id is None and "shipment_method_id" in self.model_fields_set:
            _dict['shipmentMethodId'] = None

        # set to None if salesperson (nullable) is None
        # and model_fields_set contains the field
        if self.salesperson is None and "salesperson" in self.model_fields_set:
            _dict['salesperson'] = None

        # set to None if partial_shipping (nullable) is None
        # and model_fields_set contains the field
        if self.partial_shipping is None and "partial_shipping" in self.model_fields_set:
            _dict['partialShipping'] = None

        # set to None if requested_delivery_date (nullable) is None
        # and model_fields_set contains the field
        if self.requested_delivery_date is None and "requested_delivery_date" in self.model_fields_set:
            _dict['requestedDeliveryDate'] = None

        # set to None if discount_amount (nullable) is None
        # and model_fields_set contains the field
        if self.discount_amount is None and "discount_amount" in self.model_fields_set:
            _dict['discountAmount'] = None

        # set to None if discount_applied_before_tax (nullable) is None
        # and model_fields_set contains the field
        if self.discount_applied_before_tax is None and "discount_applied_before_tax" in self.model_fields_set:
            _dict['discountAppliedBeforeTax'] = None

        # set to None if total_amount_excluding_tax (nullable) is None
        # and model_fields_set contains the field
        if self.total_amount_excluding_tax is None and "total_amount_excluding_tax" in self.model_fields_set:
            _dict['totalAmountExcludingTax'] = None

        # set to None if total_tax_amount (nullable) is None
        # and model_fields_set contains the field
        if self.total_tax_amount is None and "total_tax_amount" in self.model_fields_set:
            _dict['totalTaxAmount'] = None

        # set to None if total_amount_including_tax (nullable) is None
        # and model_fields_set contains the field
        if self.total_amount_including_tax is None and "total_amount_including_tax" in self.model_fields_set:
            _dict['totalAmountIncludingTax'] = None

        # set to None if fully_shipped (nullable) is None
        # and model_fields_set contains the field
        if self.fully_shipped is None and "fully_shipped" in self.model_fields_set:
            _dict['fullyShipped'] = None

        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict['status'] = None

        # set to None if last_modified_date_time (nullable) is None
        # and model_fields_set contains the field
        if self.last_modified_date_time is None and "last_modified_date_time" in self.model_fields_set:
            _dict['lastModifiedDateTime'] = None

        # set to None if phone_number (nullable) is None
        # and model_fields_set contains the field
        if self.phone_number is None and "phone_number" in self.model_fields_set:
            _dict['phoneNumber'] = None

        # set to None if email (nullable) is None
        # and model_fields_set contains the field
        if self.email is None and "email" in self.model_fields_set:
            _dict['email'] = None

        # set to None if sales_order_lines (nullable) is None
        # and model_fields_set contains the field
        if self.sales_order_lines is None and "sales_order_lines" in self.model_fields_set:
            _dict['salesOrderLines'] = None

        # set to None if customer (nullable) is None
        # and model_fields_set contains the field
        if self.customer is None and "customer" in self.model_fields_set:
            _dict['customer'] = None

        # set to None if currency (nullable) is None
        # and model_fields_set contains the field
        if self.currency is None and "currency" in self.model_fields_set:
            _dict['currency'] = None

        # set to None if payment_term (nullable) is None
        # and model_fields_set contains the field
        if self.payment_term is None and "payment_term" in self.model_fields_set:
            _dict['paymentTerm'] = None

        # set to None if shipment_method (nullable) is None
        # and model_fields_set contains the field
        if self.shipment_method is None and "shipment_method" in self.model_fields_set:
            _dict['shipmentMethod'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SalesOrder from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "number": obj.get("number"),
            "externalDocumentNumber": obj.get("externalDocumentNumber"),
            "orderDate": obj.get("orderDate"),
            "customerId": obj.get("customerId"),
            "contactId": obj.get("contactId"),
            "customerNumber": obj.get("customerNumber"),
            "customerName": obj.get("customerName"),
            "billToName": obj.get("billToName"),
            "billToCustomerId": obj.get("billToCustomerId"),
            "billToCustomerNumber": obj.get("billToCustomerNumber"),
            "shipToName": obj.get("shipToName"),
            "shipToContact": obj.get("shipToContact"),
            "sellingPostalAddress": Postaladdresstype.from_dict(obj["sellingPostalAddress"]) if obj.get("sellingPostalAddress") is not None else None,
            "billingPostalAddress": Postaladdresstype.from_dict(obj["billingPostalAddress"]) if obj.get("billingPostalAddress") is not None else None,
            "shippingPostalAddress": Postaladdresstype.from_dict(obj["shippingPostalAddress"]) if obj.get("shippingPostalAddress") is not None else None,
            "currencyId": obj.get("currencyId"),
            "currencyCode": obj.get("currencyCode"),
            "pricesIncludeTax": obj.get("pricesIncludeTax"),
            "paymentTermsId": obj.get("paymentTermsId"),
            "shipmentMethodId": obj.get("shipmentMethodId"),
            "salesperson": obj.get("salesperson"),
            "partialShipping": obj.get("partialShipping"),
            "requestedDeliveryDate": obj.get("requestedDeliveryDate"),
            "discountAmount": obj.get("discountAmount"),
            "discountAppliedBeforeTax": obj.get("discountAppliedBeforeTax"),
            "totalAmountExcludingTax": obj.get("totalAmountExcludingTax"),
            "totalTaxAmount": obj.get("totalTaxAmount"),
            "totalAmountIncludingTax": obj.get("totalAmountIncludingTax"),
            "fullyShipped": obj.get("fullyShipped"),
            "status": obj.get("status"),
            "lastModifiedDateTime": obj.get("lastModifiedDateTime"),
            "phoneNumber": obj.get("phoneNumber"),
            "email": obj.get("email"),
            "salesOrderLines": [SalesOrderLine.from_dict(_item) for _item in obj["salesOrderLines"]] if obj.get("salesOrderLines") is not None else None,
            "customer": Customer.from_dict(obj["customer"]) if obj.get("customer") is not None else None,
            "currency": Currency.from_dict(obj["currency"]) if obj.get("currency") is not None else None,
            "paymentTerm": PaymentTerm.from_dict(obj["paymentTerm"]) if obj.get("paymentTerm") is not None else None,
            "shipmentMethod": ShipmentMethod.from_dict(obj["shipmentMethod"]) if obj.get("shipmentMethod") is not None else None
        })
        return _obj





# coding: utf-8

"""
    (v2.0) Dynamics 365 Business Central

    (v2.0) Business Central Standard APIs

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from pybusinesscentral.model.sales_quote import SalesQuote

class TestSalesQuote(unittest.TestCase):
    """SalesQuote unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> SalesQuote:
        """Test SalesQuote
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `SalesQuote`
        """
        model = SalesQuote()
        if include_optional:
            return SalesQuote(
                id = '',
                number = '',
                external_document_number = '',
                document_date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                due_date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                customer_id = '',
                contact_id = '',
                customer_number = '',
                customer_name = '',
                bill_to_name = '',
                bill_to_customer_id = '',
                bill_to_customer_number = '',
                ship_to_name = '',
                ship_to_contact = '',
                selling_postal_address = pybusinesscentral.model.postaladdresstype.postaladdresstype(
                    street = '', 
                    city = '', 
                    state = '', 
                    country_letter_code = '', 
                    postal_code = '', 
                    customer_financial_details = [
                        pybusinesscentral.model.customer_financial_detail.customerFinancialDetail(
                            id = '', 
                            number = '', 
                            balance = 1.337, 
                            total_sales_excluding_tax = 1.337, 
                            overdue_amount = 1.337, )
                        ], 
                    picture = [
                        pybusinesscentral.model.picture.picture(
                            id = '', 
                            width = 56, 
                            height = 56, 
                            content_type = '', 
                            content@odata/media_edit_link = '', 
                            content@odata/media_read_link = '', )
                        ], 
                    default_dimensions = [
                        pybusinesscentral.model.default_dimensions.defaultDimensions(
                            parent_id = '', 
                            dimension_id = '', 
                            dimension_code = '', 
                            dimension_value_id = '', 
                            dimension_value_code = '', 
                            posting_validation = '', 
                            account = pybusinesscentral.model.account.account(
                                id = '', 
                                number = '', 
                                display_name = '', 
                                category = '', 
                                sub_category = '', 
                                blocked = True, 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                            dimension = pybusinesscentral.model.dimension.dimension(
                                id = '', 
                                code = '', 
                                display_name = '', 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                dimension_values = [
                                    pybusinesscentral.model.dimension_value.dimensionValue(
                                        id = '', 
                                        code = '', 
                                        display_name = '', 
                                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), )
                                    ], ), 
                            dimension_value = pybusinesscentral.model.dimension_value.dimensionValue(
                                id = '', 
                                code = '', 
                                display_name = '', 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), )
                        ], 
                    currency = pybusinesscentral.model.currency.currency(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        symbol = '', 
                        amount_decimal_places = '', 
                        amount_rounding_precision = 1.337, 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                    payment_term = pybusinesscentral.model.payment_term.paymentTerm(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        due_date_calculation = '', 
                        discount_date_calculation = '', 
                        discount_percent = 1.337, 
                        calculate_discount_on_credit_memos = True, 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                    shipment_method = pybusinesscentral.model.shipment_method.shipmentMethod(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                    payment_method = pybusinesscentral.model.payment_method.paymentMethod(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), ),
                billing_postal_address = pybusinesscentral.model.postaladdresstype.postaladdresstype(
                    street = '', 
                    city = '', 
                    state = '', 
                    country_letter_code = '', 
                    postal_code = '', 
                    customer_financial_details = [
                        pybusinesscentral.model.customer_financial_detail.customerFinancialDetail(
                            id = '', 
                            number = '', 
                            balance = 1.337, 
                            total_sales_excluding_tax = 1.337, 
                            overdue_amount = 1.337, )
                        ], 
                    picture = [
                        pybusinesscentral.model.picture.picture(
                            id = '', 
                            width = 56, 
                            height = 56, 
                            content_type = '', 
                            content@odata/media_edit_link = '', 
                            content@odata/media_read_link = '', )
                        ], 
                    default_dimensions = [
                        pybusinesscentral.model.default_dimensions.defaultDimensions(
                            parent_id = '', 
                            dimension_id = '', 
                            dimension_code = '', 
                            dimension_value_id = '', 
                            dimension_value_code = '', 
                            posting_validation = '', 
                            account = pybusinesscentral.model.account.account(
                                id = '', 
                                number = '', 
                                display_name = '', 
                                category = '', 
                                sub_category = '', 
                                blocked = True, 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                            dimension = pybusinesscentral.model.dimension.dimension(
                                id = '', 
                                code = '', 
                                display_name = '', 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                dimension_values = [
                                    pybusinesscentral.model.dimension_value.dimensionValue(
                                        id = '', 
                                        code = '', 
                                        display_name = '', 
                                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), )
                                    ], ), 
                            dimension_value = pybusinesscentral.model.dimension_value.dimensionValue(
                                id = '', 
                                code = '', 
                                display_name = '', 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), )
                        ], 
                    currency = pybusinesscentral.model.currency.currency(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        symbol = '', 
                        amount_decimal_places = '', 
                        amount_rounding_precision = 1.337, 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                    payment_term = pybusinesscentral.model.payment_term.paymentTerm(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        due_date_calculation = '', 
                        discount_date_calculation = '', 
                        discount_percent = 1.337, 
                        calculate_discount_on_credit_memos = True, 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                    shipment_method = pybusinesscentral.model.shipment_method.shipmentMethod(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                    payment_method = pybusinesscentral.model.payment_method.paymentMethod(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), ),
                shipping_postal_address = pybusinesscentral.model.postaladdresstype.postaladdresstype(
                    street = '', 
                    city = '', 
                    state = '', 
                    country_letter_code = '', 
                    postal_code = '', 
                    customer_financial_details = [
                        pybusinesscentral.model.customer_financial_detail.customerFinancialDetail(
                            id = '', 
                            number = '', 
                            balance = 1.337, 
                            total_sales_excluding_tax = 1.337, 
                            overdue_amount = 1.337, )
                        ], 
                    picture = [
                        pybusinesscentral.model.picture.picture(
                            id = '', 
                            width = 56, 
                            height = 56, 
                            content_type = '', 
                            content@odata/media_edit_link = '', 
                            content@odata/media_read_link = '', )
                        ], 
                    default_dimensions = [
                        pybusinesscentral.model.default_dimensions.defaultDimensions(
                            parent_id = '', 
                            dimension_id = '', 
                            dimension_code = '', 
                            dimension_value_id = '', 
                            dimension_value_code = '', 
                            posting_validation = '', 
                            account = pybusinesscentral.model.account.account(
                                id = '', 
                                number = '', 
                                display_name = '', 
                                category = '', 
                                sub_category = '', 
                                blocked = True, 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                            dimension = pybusinesscentral.model.dimension.dimension(
                                id = '', 
                                code = '', 
                                display_name = '', 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                dimension_values = [
                                    pybusinesscentral.model.dimension_value.dimensionValue(
                                        id = '', 
                                        code = '', 
                                        display_name = '', 
                                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), )
                                    ], ), 
                            dimension_value = pybusinesscentral.model.dimension_value.dimensionValue(
                                id = '', 
                                code = '', 
                                display_name = '', 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), )
                        ], 
                    currency = pybusinesscentral.model.currency.currency(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        symbol = '', 
                        amount_decimal_places = '', 
                        amount_rounding_precision = 1.337, 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                    payment_term = pybusinesscentral.model.payment_term.paymentTerm(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        due_date_calculation = '', 
                        discount_date_calculation = '', 
                        discount_percent = 1.337, 
                        calculate_discount_on_credit_memos = True, 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                    shipment_method = pybusinesscentral.model.shipment_method.shipmentMethod(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                    payment_method = pybusinesscentral.model.payment_method.paymentMethod(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), ),
                currency_id = '',
                currency_code = '',
                payment_terms_id = '',
                shipment_method_id = '',
                salesperson = '',
                discount_amount = 1.337,
                total_amount_excluding_tax = 1.337,
                total_tax_amount = 1.337,
                total_amount_including_tax = 1.337,
                status = '',
                sent_date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                valid_until_date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                accepted_date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                phone_number = '',
                email = '',
                sales_quote_lines = [
                    pybusinesscentral.model.sales_quote_line.salesQuoteLine(
                        id = '', 
                        document_id = '', 
                        sequence = 56, 
                        item_id = '', 
                        account_id = '', 
                        line_type = '', 
                        line_details = pybusinesscentral.model.documentlineobjectdetailstype.documentlineobjectdetailstype(
                            number = '', 
                            display_name = '', 
                            item = pybusinesscentral.model.item.item(
                                id = '', 
                                number = '', 
                                display_name = '', 
                                type = '', 
                                item_category_id = '', 
                                item_category_code = '', 
                                blocked = True, 
                                base_unit_of_measure_id = '', 
                                base_unit_of_measure = pybusinesscentral.model.unitofmeasuretype.unitofmeasuretype(
                                    code = '', 
                                    display_name = '', 
                                    symbol = '', 
                                    unit_conversion = pybusinesscentral.model.itemunitofmeasureconversiontype.itemunitofmeasureconversiontype(
                                        to_unit_of_measure = '', 
                                        from_to_conversion_rate = 1.337, 
                                        picture = [
                                            pybusinesscentral.model.picture.picture(
                                                id = '', 
                                                width = 56, 
                                                height = 56, 
                                                content_type = '', 
                                                content@odata/media_edit_link = '', 
                                                content@odata/media_read_link = '', )
                                            ], 
                                        default_dimensions = [
                                            pybusinesscentral.model.default_dimensions.defaultDimensions(
                                                parent_id = '', 
                                                dimension_id = '', 
                                                dimension_code = '', 
                                                dimension_value_id = '', 
                                                dimension_value_code = '', 
                                                posting_validation = '', 
                                                account = pybusinesscentral.model.account.account(
                                                    id = '', 
                                                    number = '', 
                                                    display_name = '', 
                                                    category = '', 
                                                    sub_category = '', 
                                                    blocked = True, 
                                                    last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                                                dimension = pybusinesscentral.model.dimension.dimension(
                                                    id = '', 
                                                    code = '', 
                                                    display_name = '', 
                                                    last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                                    dimension_values = [
                                                        pybusinesscentral.model.dimension_value.dimensionValue(
                                                            id = '', 
                                                            code = '', 
                                                            display_name = '', 
                                                            last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), )
                                                        ], ), 
                                                dimension_value = pybusinesscentral.model.dimension_value.dimensionValue(
                                                    id = '', 
                                                    code = '', 
                                                    display_name = '', 
                                                    last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), )
                                            ], 
                                        item_category = pybusinesscentral.model.item_category.itemCategory(
                                            id = '', 
                                            code = '', 
                                            display_name = '', 
                                            last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), ), 
                                    picture = [
                                        pybusinesscentral.model.picture.picture(
                                            id = '', 
                                            width = 56, 
                                            height = 56, 
                                            content_type = '', 
                                            content@odata/media_edit_link = '', 
                                            content@odata/media_read_link = '', )
                                        ], 
                                    default_dimensions = [
                                        pybusinesscentral.model.default_dimensions.defaultDimensions(
                                            parent_id = '', 
                                            dimension_id = '', 
                                            dimension_code = '', 
                                            dimension_value_id = '', 
                                            dimension_value_code = '', 
                                            posting_validation = '', )
                                        ], 
                                    item_category = pybusinesscentral.model.item_category.itemCategory(
                                        id = '', 
                                        code = '', 
                                        display_name = '', 
                                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), ), 
                                gtin = '', 
                                inventory = 1.337, 
                                unit_price = 1.337, 
                                price_includes_tax = True, 
                                unit_cost = 1.337, 
                                tax_group_id = '', 
                                tax_group_code = '', 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                picture = , 
                                default_dimensions = , 
                                item_category = , ), 
                            account = pybusinesscentral.model.account.account(
                                id = '', 
                                number = '', 
                                display_name = '', 
                                category = '', 
                                sub_category = '', 
                                blocked = True, 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), ), 
                        description = '', 
                        unit_of_measure_id = '', 
                        unit_of_measure = pybusinesscentral.model.unitofmeasuretype.unitofmeasuretype(
                            code = '', 
                            display_name = '', 
                            symbol = '', ), 
                        unit_price = 1.337, 
                        quantity = 1.337, 
                        discount_amount = 1.337, 
                        discount_percent = 1.337, 
                        discount_applied_before_tax = True, 
                        amount_excluding_tax = 1.337, 
                        tax_code = '', 
                        tax_percent = 1.337, 
                        total_tax_amount = 1.337, 
                        amount_including_tax = 1.337, 
                        net_amount = 1.337, 
                        net_tax_amount = 1.337, 
                        net_amount_including_tax = 1.337, 
                        item = pybusinesscentral.model.item.item(
                            id = '', 
                            number = '', 
                            display_name = '', 
                            type = '', 
                            item_category_id = '', 
                            item_category_code = '', 
                            blocked = True, 
                            base_unit_of_measure_id = '', 
                            gtin = '', 
                            inventory = 1.337, 
                            unit_price = 1.337, 
                            price_includes_tax = True, 
                            unit_cost = 1.337, 
                            tax_group_id = '', 
                            tax_group_code = '', 
                            last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                        account = , )
                    ],
                pdf_document = [
                    pybusinesscentral.model.pdf_document.pdfDocument(
                        id = '', 
                        content = bytes(b'blah'), )
                    ],
                customer = pybusinesscentral.model.customer.customer(
                    id = '', 
                    number = '', 
                    display_name = '', 
                    type = '', 
                    address_line1 = '', 
                    address_line2 = '', 
                    city = '', 
                    state = '', 
                    country = '', 
                    postal_code = '', 
                    phone_number = '', 
                    email = '', 
                    website = '', 
                    tax_liable = True, 
                    tax_area_id = '', 
                    tax_area_display_name = '', 
                    tax_registration_number = '', 
                    currency_id = '', 
                    currency_code = '', 
                    payment_terms_id = '', 
                    shipment_method_id = '', 
                    payment_method_id = '', 
                    blocked = '', 
                    last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                    customer_financial_details = [
                        pybusinesscentral.model.customer_financial_detail.customerFinancialDetail(
                            id = '', 
                            number = '', 
                            balance = 1.337, 
                            total_sales_excluding_tax = 1.337, 
                            overdue_amount = 1.337, )
                        ], 
                    picture = [
                        pybusinesscentral.model.picture.picture(
                            id = '', 
                            width = 56, 
                            height = 56, 
                            content_type = '', 
                            content@odata/media_edit_link = '', 
                            content@odata/media_read_link = '', )
                        ], 
                    default_dimensions = [
                        pybusinesscentral.model.default_dimensions.defaultDimensions(
                            parent_id = '', 
                            dimension_id = '', 
                            dimension_code = '', 
                            dimension_value_id = '', 
                            dimension_value_code = '', 
                            posting_validation = '', 
                            account = pybusinesscentral.model.account.account(
                                id = '', 
                                number = '', 
                                display_name = '', 
                                category = '', 
                                sub_category = '', 
                                blocked = True, 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                            dimension = pybusinesscentral.model.dimension.dimension(
                                id = '', 
                                code = '', 
                                display_name = '', 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                dimension_values = [
                                    pybusinesscentral.model.dimension_value.dimensionValue(
                                        id = '', 
                                        code = '', 
                                        display_name = '', 
                                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), )
                                    ], ), 
                            dimension_value = pybusinesscentral.model.dimension_value.dimensionValue(
                                id = '', 
                                code = '', 
                                display_name = '', 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), )
                        ], 
                    currency = pybusinesscentral.model.currency.currency(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        symbol = '', 
                        amount_decimal_places = '', 
                        amount_rounding_precision = 1.337, 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                    payment_term = pybusinesscentral.model.payment_term.paymentTerm(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        due_date_calculation = '', 
                        discount_date_calculation = '', 
                        discount_percent = 1.337, 
                        calculate_discount_on_credit_memos = True, 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                    shipment_method = pybusinesscentral.model.shipment_method.shipmentMethod(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                    payment_method = pybusinesscentral.model.payment_method.paymentMethod(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), ),
                currency = pybusinesscentral.model.currency.currency(
                    id = '', 
                    code = '', 
                    display_name = '', 
                    symbol = '', 
                    amount_decimal_places = '', 
                    amount_rounding_precision = 1.337, 
                    last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ),
                payment_term = pybusinesscentral.model.payment_term.paymentTerm(
                    id = '', 
                    code = '', 
                    display_name = '', 
                    due_date_calculation = '', 
                    discount_date_calculation = '', 
                    discount_percent = 1.337, 
                    calculate_discount_on_credit_memos = True, 
                    last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ),
                shipment_method = pybusinesscentral.model.shipment_method.shipmentMethod(
                    id = '', 
                    code = '', 
                    display_name = '', 
                    last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), )
            )
        else:
            return SalesQuote(
        )
        """

    def testSalesQuote(self):
        """Test SalesQuote"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

# flake8: noqa

"""
    (v2.0) Dynamics 365 Business Central

    (v2.0) Business Central Standard APIs

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# import apis into sdk package
from pybusinesscentral.api.account_api import AccountApi
from pybusinesscentral.api.company_api import CompanyApi
from pybusinesscentral.api.customer_api import CustomerApi
from pybusinesscentral.api.dimension_api import DimensionApi
from pybusinesscentral.api.dimension_set_line_api import DimensionSetLineApi
from pybusinesscentral.api.payment_term_api import PaymentTermApi
from pybusinesscentral.api.purchase_invoice_api import PurchaseInvoiceApi
from pybusinesscentral.api.sales_credit_memo_api import SalesCreditMemoApi
from pybusinesscentral.api.sales_credit_memo_line_api import SalesCreditMemoLineApi
from pybusinesscentral.api.sales_invoice_api import SalesInvoiceApi
from pybusinesscentral.api.sales_invoice_line_api import SalesInvoiceLineApi
from pybusinesscentral.api.vendor_api import VendorApi

# import ApiClient
from pybusinesscentral.api_response import ApiResponse
from pybusinesscentral.api_client import ApiClient
from pybusinesscentral.configuration import Configuration
from pybusinesscentral.exceptions import OpenApiException
from pybusinesscentral.exceptions import ApiTypeError
from pybusinesscentral.exceptions import ApiValueError
from pybusinesscentral.exceptions import ApiKeyError
from pybusinesscentral.exceptions import ApiAttributeError
from pybusinesscentral.exceptions import ApiException

# import models into sdk package
from pybusinesscentral.model.account import Account
from pybusinesscentral.model.aged_accounts_payable import AgedAccountsPayable
from pybusinesscentral.model.aged_accounts_receivable import AgedAccountsReceivable
from pybusinesscentral.model.attachments import Attachments
from pybusinesscentral.model.balance_sheet import BalanceSheet
from pybusinesscentral.model.bank_account import BankAccount
from pybusinesscentral.model.cash_flow_statement import CashFlowStatement
from pybusinesscentral.model.company import Company
from pybusinesscentral.model.company_information import CompanyInformation
from pybusinesscentral.model.country_region import CountryRegion
from pybusinesscentral.model.currency import Currency
from pybusinesscentral.model.customer import Customer
from pybusinesscentral.model.customer_financial_detail import CustomerFinancialDetail
from pybusinesscentral.model.customer_payment import CustomerPayment
from pybusinesscentral.model.customer_payment_journal import CustomerPaymentJournal
from pybusinesscentral.model.customer_sale import CustomerSale
from pybusinesscentral.model.default_dimensions import DefaultDimensions
from pybusinesscentral.model.dimension import Dimension
from pybusinesscentral.model.dimension_line import DimensionLine
from pybusinesscentral.model.dimension_set_line import DimensionSetLine
from pybusinesscentral.model.dimension_value import DimensionValue
from pybusinesscentral.model.dimensiontype import Dimensiontype
from pybusinesscentral.model.documentlineobjectdetailstype import Documentlineobjectdetailstype
from pybusinesscentral.model.employee import Employee
from pybusinesscentral.model.general_ledger_entry import GeneralLedgerEntry
from pybusinesscentral.model.general_ledger_entry_attachments import GeneralLedgerEntryAttachments
from pybusinesscentral.model.general_ledger_entry_dimensions_inner import GeneralLedgerEntryDimensionsInner
from pybusinesscentral.model.income_statement import IncomeStatement
from pybusinesscentral.model.item import Item
from pybusinesscentral.model.item_category import ItemCategory
from pybusinesscentral.model.itemunitofmeasureconversiontype import Itemunitofmeasureconversiontype
from pybusinesscentral.model.journal import Journal
from pybusinesscentral.model.journal_line import JournalLine
from pybusinesscentral.model.list_accounts200_response import ListAccounts200Response
from pybusinesscentral.model.list_companies200_response import ListCompanies200Response
from pybusinesscentral.model.list_customers200_response import ListCustomers200Response
from pybusinesscentral.model.list_dimensions200_response import ListDimensions200Response
from pybusinesscentral.model.list_payment_terms200_response import ListPaymentTerms200Response
from pybusinesscentral.model.list_purchase_invoices200_response import ListPurchaseInvoices200Response
from pybusinesscentral.model.list_sales_credit_memo_lines_for_sales_credit_memo200_response import ListSalesCreditMemoLinesForSalesCreditMemo200Response
from pybusinesscentral.model.list_sales_credit_memos200_response import ListSalesCreditMemos200Response
from pybusinesscentral.model.list_sales_invoice_lines_for_sales_invoice200_response import ListSalesInvoiceLinesForSalesInvoice200Response
from pybusinesscentral.model.list_sales_invoices200_response import ListSalesInvoices200Response
from pybusinesscentral.model.list_vendors200_response import ListVendors200Response
from pybusinesscentral.model.payment_method import PaymentMethod
from pybusinesscentral.model.payment_term import PaymentTerm
from pybusinesscentral.model.pdf_document import PdfDocument
from pybusinesscentral.model.picture import Picture
from pybusinesscentral.model.post_customer_request import PostCustomerRequest
from pybusinesscentral.model.post_dimension_set_line_request import PostDimensionSetLineRequest
from pybusinesscentral.model.post_payment_term_request import PostPaymentTermRequest
from pybusinesscentral.model.post_purchase_invoice_request import PostPurchaseInvoiceRequest
from pybusinesscentral.model.post_sales_credit_memo_line_for_sales_credit_memo_request import PostSalesCreditMemoLineForSalesCreditMemoRequest
from pybusinesscentral.model.post_sales_credit_memo_request import PostSalesCreditMemoRequest
from pybusinesscentral.model.post_sales_invoice_line_for_sales_invoice_request import PostSalesInvoiceLineForSalesInvoiceRequest
from pybusinesscentral.model.post_sales_invoice_request import PostSalesInvoiceRequest
from pybusinesscentral.model.post_vendor_request import PostVendorRequest
from pybusinesscentral.model.postaladdresstype import Postaladdresstype
from pybusinesscentral.model.project import Project
from pybusinesscentral.model.purchase_invoice import PurchaseInvoice
from pybusinesscentral.model.purchase_invoice_line import PurchaseInvoiceLine
from pybusinesscentral.model.retained_earnings_statement import RetainedEarningsStatement
from pybusinesscentral.model.sales_credit_memo import SalesCreditMemo
from pybusinesscentral.model.sales_credit_memo_line import SalesCreditMemoLine
from pybusinesscentral.model.sales_invoice import SalesInvoice
from pybusinesscentral.model.sales_invoice_line import SalesInvoiceLine
from pybusinesscentral.model.sales_order import SalesOrder
from pybusinesscentral.model.sales_order_line import SalesOrderLine
from pybusinesscentral.model.sales_quote import SalesQuote
from pybusinesscentral.model.sales_quote_line import SalesQuoteLine
from pybusinesscentral.model.shipment_method import ShipmentMethod
from pybusinesscentral.model.tax_area import TaxArea
from pybusinesscentral.model.tax_group import TaxGroup
from pybusinesscentral.model.time_registration_entry import TimeRegistrationEntry
from pybusinesscentral.model.trial_balance import TrialBalance
from pybusinesscentral.model.unit_of_measure import UnitOfMeasure
from pybusinesscentral.model.unitofmeasuretype import Unitofmeasuretype
from pybusinesscentral.model.vendor import Vendor
from pybusinesscentral.model.vendor_purchase import VendorPurchase



# coding: utf-8

"""
    (v2.0) Dynamics 365 Business Central

    (v2.0) Business Central Standard APIs

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from pybusinesscentral.model.attachments import Attachments

class TestAttachments(unittest.TestCase):
    """Attachments unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> Attachments:
        """Test Attachments
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `Attachments`
        """
        model = Attachments()
        if include_optional:
            return Attachments(
                parent_id = '',
                id = '',
                file_name = '',
                byte_size = 56,
                content = bytes(b'blah'),
                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f')
            )
        else:
            return Attachments(
        )
        """

    def testAttachments(self):
        """Test Attachments"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    (v2.0) Dynamics 365 Business Central

    (v2.0) Business Central Standard APIs

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from pybusinesscentral.model.general_ledger_entry_attachments import GeneralLedgerEntryAttachments

class TestGeneralLedgerEntryAttachments(unittest.TestCase):
    """GeneralLedgerEntryAttachments unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> GeneralLedgerEntryAttachments:
        """Test GeneralLedgerEntryAttachments
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `GeneralLedgerEntryAttachments`
        """
        model = GeneralLedgerEntryAttachments()
        if include_optional:
            return GeneralLedgerEntryAttachments(
                general_ledger_entry_number = 56,
                id = '',
                file_name = '',
                byte_size = 56,
                content = bytes(b'blah'),
                created_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                general_ledger_entry = pybusinesscentral.model.general_ledger_entry.generalLedgerEntry(
                    id = 56, 
                    posting_date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                    document_number = '', 
                    document_type = '', 
                    account_id = '', 
                    account_number = '', 
                    description = '', 
                    debit_amount = 1.337, 
                    credit_amount = 1.337, 
                    dimensions = [
                        null
                        ], 
                    last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                    account = pybusinesscentral.model.account.account(
                        id = '', 
                        number = '', 
                        display_name = '', 
                        category = '', 
                        sub_category = '', 
                        blocked = True, 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), )
            )
        else:
            return GeneralLedgerEntryAttachments(
        )
        """

    def testGeneralLedgerEntryAttachments(self):
        """Test GeneralLedgerEntryAttachments"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    (v2.0) Dynamics 365 Business Central

    (v2.0) Business Central Standard APIs

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from pybusinesscentral.model.list_sales_credit_memos200_response import ListSalesCreditMemos200Response

class TestListSalesCreditMemos200Response(unittest.TestCase):
    """ListSalesCreditMemos200Response unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ListSalesCreditMemos200Response:
        """Test ListSalesCreditMemos200Response
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ListSalesCreditMemos200Response`
        """
        model = ListSalesCreditMemos200Response()
        if include_optional:
            return ListSalesCreditMemos200Response(
                value = [
                    pybusinesscentral.model.sales_credit_memo.salesCreditMemo(
                        id = '', 
                        number = '', 
                        external_document_number = '', 
                        credit_memo_date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        due_date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        customer_id = '', 
                        contact_id = '', 
                        customer_number = '', 
                        customer_name = '', 
                        bill_to_name = '', 
                        bill_to_customer_id = '', 
                        bill_to_customer_number = '', 
                        selling_postal_address = pybusinesscentral.model.postaladdresstype.postaladdresstype(
                            street = '', 
                            city = '', 
                            state = '', 
                            country_letter_code = '', 
                            postal_code = '', 
                            customer_financial_details = [
                                pybusinesscentral.model.customer_financial_detail.customerFinancialDetail(
                                    id = '', 
                                    number = '', 
                                    balance = 1.337, 
                                    total_sales_excluding_tax = 1.337, 
                                    overdue_amount = 1.337, )
                                ], 
                            picture = [
                                pybusinesscentral.model.picture.picture(
                                    id = '', 
                                    width = 56, 
                                    height = 56, 
                                    content_type = '', 
                                    content@odata/media_edit_link = '', 
                                    content@odata/media_read_link = '', )
                                ], 
                            default_dimensions = [
                                pybusinesscentral.model.default_dimensions.defaultDimensions(
                                    parent_id = '', 
                                    dimension_id = '', 
                                    dimension_code = '', 
                                    dimension_value_id = '', 
                                    dimension_value_code = '', 
                                    posting_validation = '', 
                                    account = pybusinesscentral.model.account.account(
                                        id = '', 
                                        number = '', 
                                        display_name = '', 
                                        category = '', 
                                        sub_category = '', 
                                        blocked = True, 
                                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                                    dimension = pybusinesscentral.model.dimension.dimension(
                                        id = '', 
                                        code = '', 
                                        display_name = '', 
                                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                        dimension_values = [
                                            pybusinesscentral.model.dimension_value.dimensionValue(
                                                id = '', 
                                                code = '', 
                                                display_name = '', 
                                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), )
                                            ], ), 
                                    dimension_value = pybusinesscentral.model.dimension_value.dimensionValue(
                                        id = '', 
                                        code = '', 
                                        display_name = '', 
                                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), )
                                ], 
                            currency = pybusinesscentral.model.currency.currency(
                                id = '', 
                                code = '', 
                                display_name = '', 
                                symbol = '', 
                                amount_decimal_places = '', 
                                amount_rounding_precision = 1.337, 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                            payment_term = pybusinesscentral.model.payment_term.paymentTerm(
                                id = '', 
                                code = '', 
                                display_name = '', 
                                due_date_calculation = '', 
                                discount_date_calculation = '', 
                                discount_percent = 1.337, 
                                calculate_discount_on_credit_memos = True, 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                            shipment_method = pybusinesscentral.model.shipment_method.shipmentMethod(
                                id = '', 
                                code = '', 
                                display_name = '', 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                            payment_method = pybusinesscentral.model.payment_method.paymentMethod(
                                id = '', 
                                code = '', 
                                display_name = '', 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), ), 
                        billing_postal_address = pybusinesscentral.model.postaladdresstype.postaladdresstype(
                            street = '', 
                            city = '', 
                            state = '', 
                            country_letter_code = '', 
                            postal_code = '', ), 
                        currency_id = '', 
                        currency_code = '', 
                        payment_terms_id = '', 
                        shipment_method_id = '', 
                        salesperson = '', 
                        prices_include_tax = True, 
                        discount_amount = 1.337, 
                        discount_applied_before_tax = True, 
                        total_amount_excluding_tax = 1.337, 
                        total_tax_amount = 1.337, 
                        total_amount_including_tax = 1.337, 
                        status = '', 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        invoice_id = '', 
                        invoice_number = '', 
                        phone_number = '', 
                        email = '', 
                        sales_credit_memo_lines = [
                            pybusinesscentral.model.sales_credit_memo_line.salesCreditMemoLine(
                                id = '', 
                                document_id = '', 
                                sequence = 56, 
                                item_id = '', 
                                account_id = '', 
                                line_type = '', 
                                line_details = pybusinesscentral.model.documentlineobjectdetailstype.documentlineobjectdetailstype(
                                    number = '', 
                                    display_name = '', 
                                    item = pybusinesscentral.model.item.item(
                                        id = '', 
                                        number = '', 
                                        display_name = '', 
                                        type = '', 
                                        item_category_id = '', 
                                        item_category_code = '', 
                                        blocked = True, 
                                        base_unit_of_measure_id = '', 
                                        base_unit_of_measure = pybusinesscentral.model.unitofmeasuretype.unitofmeasuretype(
                                            code = '', 
                                            display_name = '', 
                                            symbol = '', 
                                            unit_conversion = pybusinesscentral.model.itemunitofmeasureconversiontype.itemunitofmeasureconversiontype(
                                                to_unit_of_measure = '', 
                                                from_to_conversion_rate = 1.337, 
                                                item_category = pybusinesscentral.model.item_category.itemCategory(
                                                    id = '', 
                                                    code = '', 
                                                    display_name = '', 
                                                    last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), ), 
                                            item_category = pybusinesscentral.model.item_category.itemCategory(
                                                id = '', 
                                                code = '', 
                                                display_name = '', 
                                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), ), 
                                        gtin = '', 
                                        inventory = 1.337, 
                                        unit_price = 1.337, 
                                        price_includes_tax = True, 
                                        unit_cost = 1.337, 
                                        tax_group_id = '', 
                                        tax_group_code = '', 
                                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                        item_category = , ), ), 
                                description = '', 
                                unit_of_measure_id = '', 
                                unit_of_measure = pybusinesscentral.model.unitofmeasuretype.unitofmeasuretype(
                                    code = '', 
                                    display_name = '', 
                                    symbol = '', ), 
                                unit_price = 1.337, 
                                quantity = 1.337, 
                                discount_amount = 1.337, 
                                discount_percent = 1.337, 
                                discount_applied_before_tax = True, 
                                amount_excluding_tax = 1.337, 
                                tax_code = '', 
                                tax_percent = 1.337, 
                                total_tax_amount = 1.337, 
                                amount_including_tax = 1.337, 
                                invoice_discount_allocation = 1.337, 
                                net_amount = 1.337, 
                                net_tax_amount = 1.337, 
                                net_amount_including_tax = 1.337, 
                                shipment_date = datetime.datetime.strptime('1975-12-30', '%Y-%m-%d').date(), 
                                item = pybusinesscentral.model.item.item(
                                    id = '', 
                                    number = '', 
                                    display_name = '', 
                                    type = '', 
                                    item_category_id = '', 
                                    item_category_code = '', 
                                    blocked = True, 
                                    base_unit_of_measure_id = '', 
                                    gtin = '', 
                                    inventory = 1.337, 
                                    unit_price = 1.337, 
                                    price_includes_tax = True, 
                                    unit_cost = 1.337, 
                                    tax_group_id = '', 
                                    tax_group_code = '', 
                                    last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), )
                            ], 
                        pdf_document = [
                            pybusinesscentral.model.pdf_document.pdfDocument(
                                id = '', 
                                content = bytes(b'blah'), )
                            ], 
                        customer = pybusinesscentral.model.customer.customer(
                            id = '', 
                            number = '', 
                            display_name = '', 
                            type = '', 
                            address_line1 = '', 
                            address_line2 = '', 
                            city = '', 
                            state = '', 
                            country = '', 
                            postal_code = '', 
                            phone_number = '', 
                            email = '', 
                            website = '', 
                            tax_liable = True, 
                            tax_area_id = '', 
                            tax_area_display_name = '', 
                            tax_registration_number = '', 
                            currency_id = '', 
                            currency_code = '', 
                            payment_terms_id = '', 
                            shipment_method_id = '', 
                            payment_method_id = '', 
                            blocked = '', 
                            last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                        currency = pybusinesscentral.model.currency.currency(
                            id = '', 
                            code = '', 
                            display_name = '', 
                            symbol = '', 
                            amount_decimal_places = '', 
                            amount_rounding_precision = 1.337, 
                            last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                        payment_term = pybusinesscentral.model.payment_term.paymentTerm(
                            id = '', 
                            code = '', 
                            display_name = '', 
                            due_date_calculation = '', 
                            discount_date_calculation = '', 
                            discount_percent = 1.337, 
                            calculate_discount_on_credit_memos = True, 
                            last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                        shipment_method = pybusinesscentral.model.shipment_method.shipmentMethod(
                            id = '', 
                            code = '', 
                            display_name = '', 
                            last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), )
                    ]
            )
        else:
            return ListSalesCreditMemos200Response(
        )
        """

    def testListSalesCreditMemos200Response(self):
        """Test ListSalesCreditMemos200Response"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    (v2.0) Dynamics 365 Business Central

    (v2.0) Business Central Standard APIs

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from pybusinesscentral.model.account import Account
from typing import Optional, Set
from typing_extensions import Self

class TrialBalance(BaseModel):
    """
    TrialBalance
    """ # noqa: E501
    number: Optional[Annotated[str, Field(strict=True, max_length=20)]] = Field(default=None, description="(v1.0) The number property for the Dynamics 365 Business Central trialBalance entity")
    account_id: Optional[StrictStr] = Field(default=None, description="(v1.0) The accountId property for the Dynamics 365 Business Central trialBalance entity", alias="accountId")
    account_type: Optional[StrictStr] = Field(default=None, description="(v1.0) The accountType property for the Dynamics 365 Business Central trialBalance entity", alias="accountType")
    display: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="(v1.0) The display property for the Dynamics 365 Business Central trialBalance entity")
    total_debit: Optional[Annotated[str, Field(strict=True, max_length=30)]] = Field(default=None, description="(v1.0) The totalDebit property for the Dynamics 365 Business Central trialBalance entity", alias="totalDebit")
    total_credit: Optional[Annotated[str, Field(strict=True, max_length=30)]] = Field(default=None, description="(v1.0) The totalCredit property for the Dynamics 365 Business Central trialBalance entity", alias="totalCredit")
    balance_at_date_debit: Optional[Annotated[str, Field(strict=True, max_length=30)]] = Field(default=None, description="(v1.0) The balanceAtDateDebit property for the Dynamics 365 Business Central trialBalance entity", alias="balanceAtDateDebit")
    balance_at_date_credit: Optional[Annotated[str, Field(strict=True, max_length=30)]] = Field(default=None, description="(v1.0) The balanceAtDateCredit property for the Dynamics 365 Business Central trialBalance entity", alias="balanceAtDateCredit")
    date_filter: Optional[datetime] = Field(default=None, description="(v1.0) The dateFilter property for the Dynamics 365 Business Central trialBalance entity", alias="dateFilter")
    account: Optional[Account] = None
    __properties: ClassVar[List[str]] = ["number", "accountId", "accountType", "display", "totalDebit", "totalCredit", "balanceAtDateDebit", "balanceAtDateCredit", "dateFilter", "account"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of TrialBalance from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of account
        if self.account:
            _dict['account'] = self.account.to_dict()
        # set to None if account_id (nullable) is None
        # and model_fields_set contains the field
        if self.account_id is None and "account_id" in self.model_fields_set:
            _dict['accountId'] = None

        # set to None if account_type (nullable) is None
        # and model_fields_set contains the field
        if self.account_type is None and "account_type" in self.model_fields_set:
            _dict['accountType'] = None

        # set to None if display (nullable) is None
        # and model_fields_set contains the field
        if self.display is None and "display" in self.model_fields_set:
            _dict['display'] = None

        # set to None if total_debit (nullable) is None
        # and model_fields_set contains the field
        if self.total_debit is None and "total_debit" in self.model_fields_set:
            _dict['totalDebit'] = None

        # set to None if total_credit (nullable) is None
        # and model_fields_set contains the field
        if self.total_credit is None and "total_credit" in self.model_fields_set:
            _dict['totalCredit'] = None

        # set to None if balance_at_date_debit (nullable) is None
        # and model_fields_set contains the field
        if self.balance_at_date_debit is None and "balance_at_date_debit" in self.model_fields_set:
            _dict['balanceAtDateDebit'] = None

        # set to None if balance_at_date_credit (nullable) is None
        # and model_fields_set contains the field
        if self.balance_at_date_credit is None and "balance_at_date_credit" in self.model_fields_set:
            _dict['balanceAtDateCredit'] = None

        # set to None if date_filter (nullable) is None
        # and model_fields_set contains the field
        if self.date_filter is None and "date_filter" in self.model_fields_set:
            _dict['dateFilter'] = None

        # set to None if account (nullable) is None
        # and model_fields_set contains the field
        if self.account is None and "account" in self.model_fields_set:
            _dict['account'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of TrialBalance from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "number": obj.get("number"),
            "accountId": obj.get("accountId"),
            "accountType": obj.get("accountType"),
            "display": obj.get("display"),
            "totalDebit": obj.get("totalDebit"),
            "totalCredit": obj.get("totalCredit"),
            "balanceAtDateDebit": obj.get("balanceAtDateDebit"),
            "balanceAtDateCredit": obj.get("balanceAtDateCredit"),
            "dateFilter": obj.get("dateFilter"),
            "account": Account.from_dict(obj["account"]) if obj.get("account") is not None else None
        })
        return _obj





# coding: utf-8

"""
    (v2.0) Dynamics 365 Business Central

    (v2.0) Business Central Standard APIs

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from pybusinesscentral.model.picture import Picture

class TestPicture(unittest.TestCase):
    """Picture unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> Picture:
        """Test Picture
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `Picture`
        """
        model = Picture()
        if include_optional:
            return Picture(
                id = '',
                width = 56,
                height = 56,
                content_type = '',
                contentodata_media_edit_link = '',
                contentodata_media_read_link = ''
            )
        else:
            return Picture(
        )
        """

    def testPicture(self):
        """Test Picture"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    (v2.0) Dynamics 365 Business Central

    (v2.0) Business Central Standard APIs

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from pybusinesscentral.model.post_sales_invoice_line_for_sales_invoice_request import PostSalesInvoiceLineForSalesInvoiceRequest

class TestPostSalesInvoiceLineForSalesInvoiceRequest(unittest.TestCase):
    """PostSalesInvoiceLineForSalesInvoiceRequest unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> PostSalesInvoiceLineForSalesInvoiceRequest:
        """Test PostSalesInvoiceLineForSalesInvoiceRequest
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `PostSalesInvoiceLineForSalesInvoiceRequest`
        """
        model = PostSalesInvoiceLineForSalesInvoiceRequest()
        if include_optional:
            return PostSalesInvoiceLineForSalesInvoiceRequest(
                id = '',
                document_id = '',
                sequence = 56,
                item_id = '',
                account_id = '',
                line_type = '',
                line_details = pybusinesscentral.model.documentlineobjectdetailstype.documentlineobjectdetailstype(
                    number = '', 
                    display_name = '', 
                    item = pybusinesscentral.model.item.item(
                        id = '', 
                        number = '', 
                        display_name = '', 
                        type = '', 
                        item_category_id = '', 
                        item_category_code = '', 
                        blocked = True, 
                        base_unit_of_measure_id = '', 
                        base_unit_of_measure = pybusinesscentral.model.unitofmeasuretype.unitofmeasuretype(
                            code = '', 
                            display_name = '', 
                            symbol = '', 
                            unit_conversion = pybusinesscentral.model.itemunitofmeasureconversiontype.itemunitofmeasureconversiontype(
                                to_unit_of_measure = '', 
                                from_to_conversion_rate = 1.337, 
                                picture = [
                                    pybusinesscentral.model.picture.picture(
                                        id = '', 
                                        width = 56, 
                                        height = 56, 
                                        content_type = '', 
                                        content@odata/media_edit_link = '', 
                                        content@odata/media_read_link = '', )
                                    ], 
                                default_dimensions = [
                                    pybusinesscentral.model.default_dimensions.defaultDimensions(
                                        parent_id = '', 
                                        dimension_id = '', 
                                        dimension_code = '', 
                                        dimension_value_id = '', 
                                        dimension_value_code = '', 
                                        posting_validation = '', 
                                        account = pybusinesscentral.model.account.account(
                                            id = '', 
                                            number = '', 
                                            display_name = '', 
                                            category = '', 
                                            sub_category = '', 
                                            blocked = True, 
                                            last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                                        dimension = pybusinesscentral.model.dimension.dimension(
                                            id = '', 
                                            code = '', 
                                            display_name = '', 
                                            last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                            dimension_values = [
                                                pybusinesscentral.model.dimension_value.dimensionValue(
                                                    id = '', 
                                                    code = '', 
                                                    display_name = '', 
                                                    last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), )
                                                ], ), 
                                        dimension_value = pybusinesscentral.model.dimension_value.dimensionValue(
                                            id = '', 
                                            code = '', 
                                            display_name = '', 
                                            last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), )
                                    ], 
                                item_category = pybusinesscentral.model.item_category.itemCategory(
                                    id = '', 
                                    code = '', 
                                    display_name = '', 
                                    last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), ), 
                            picture = [
                                pybusinesscentral.model.picture.picture(
                                    id = '', 
                                    width = 56, 
                                    height = 56, 
                                    content_type = '', 
                                    content@odata/media_edit_link = '', 
                                    content@odata/media_read_link = '', )
                                ], 
                            default_dimensions = [
                                pybusinesscentral.model.default_dimensions.defaultDimensions(
                                    parent_id = '', 
                                    dimension_id = '', 
                                    dimension_code = '', 
                                    dimension_value_id = '', 
                                    dimension_value_code = '', 
                                    posting_validation = '', )
                                ], 
                            item_category = pybusinesscentral.model.item_category.itemCategory(
                                id = '', 
                                code = '', 
                                display_name = '', 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), ), 
                        gtin = '', 
                        inventory = 1.337, 
                        unit_price = 1.337, 
                        price_includes_tax = True, 
                        unit_cost = 1.337, 
                        tax_group_id = '', 
                        tax_group_code = '', 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        picture = , 
                        default_dimensions = , 
                        item_category = , ), 
                    account = pybusinesscentral.model.account.account(
                        id = '', 
                        number = '', 
                        display_name = '', 
                        category = '', 
                        sub_category = '', 
                        blocked = True, 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), ),
                description = '',
                unit_of_measure_id = '',
                unit_of_measure = pybusinesscentral.model.unitofmeasuretype.unitofmeasuretype(
                    code = '', 
                    display_name = '', 
                    symbol = '', 
                    unit_conversion = pybusinesscentral.model.itemunitofmeasureconversiontype.itemunitofmeasureconversiontype(
                        to_unit_of_measure = '', 
                        from_to_conversion_rate = 1.337, 
                        picture = [
                            pybusinesscentral.model.picture.picture(
                                id = '', 
                                width = 56, 
                                height = 56, 
                                content_type = '', 
                                content@odata/media_edit_link = '', 
                                content@odata/media_read_link = '', )
                            ], 
                        default_dimensions = [
                            pybusinesscentral.model.default_dimensions.defaultDimensions(
                                parent_id = '', 
                                dimension_id = '', 
                                dimension_code = '', 
                                dimension_value_id = '', 
                                dimension_value_code = '', 
                                posting_validation = '', 
                                account = pybusinesscentral.model.account.account(
                                    id = '', 
                                    number = '', 
                                    display_name = '', 
                                    category = '', 
                                    sub_category = '', 
                                    blocked = True, 
                                    last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                                dimension = pybusinesscentral.model.dimension.dimension(
                                    id = '', 
                                    code = '', 
                                    display_name = '', 
                                    last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                    dimension_values = [
                                        pybusinesscentral.model.dimension_value.dimensionValue(
                                            id = '', 
                                            code = '', 
                                            display_name = '', 
                                            last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), )
                                        ], ), 
                                dimension_value = pybusinesscentral.model.dimension_value.dimensionValue(
                                    id = '', 
                                    code = '', 
                                    display_name = '', 
                                    last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), )
                            ], 
                        item_category = pybusinesscentral.model.item_category.itemCategory(
                            id = '', 
                            code = '', 
                            display_name = '', 
                            last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), ), 
                    picture = [
                        pybusinesscentral.model.picture.picture(
                            id = '', 
                            width = 56, 
                            height = 56, 
                            content_type = '', 
                            content@odata/media_edit_link = '', 
                            content@odata/media_read_link = '', )
                        ], 
                    default_dimensions = [
                        pybusinesscentral.model.default_dimensions.defaultDimensions(
                            parent_id = '', 
                            dimension_id = '', 
                            dimension_code = '', 
                            dimension_value_id = '', 
                            dimension_value_code = '', 
                            posting_validation = '', )
                        ], 
                    item_category = pybusinesscentral.model.item_category.itemCategory(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), ),
                unit_price = 1.337,
                quantity = 1.337,
                discount_amount = 1.337,
                discount_percent = 1.337,
                discount_applied_before_tax = True,
                amount_excluding_tax = 1.337,
                tax_code = '',
                tax_percent = 1.337,
                total_tax_amount = 1.337,
                amount_including_tax = 1.337,
                invoice_discount_allocation = 1.337,
                net_amount = 1.337,
                net_tax_amount = 1.337,
                net_amount_including_tax = 1.337,
                shipment_date = datetime.datetime.strptime('1975-12-30', '%Y-%m-%d').date()
            )
        else:
            return PostSalesInvoiceLineForSalesInvoiceRequest(
        )
        """

    def testPostSalesInvoiceLineForSalesInvoiceRequest(self):
        """Test PostSalesInvoiceLineForSalesInvoiceRequest"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    (v2.0) Dynamics 365 Business Central

    (v2.0) Business Central Standard APIs

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from pybusinesscentral.model.journal import Journal

class TestJournal(unittest.TestCase):
    """Journal unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> Journal:
        """Test Journal
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `Journal`
        """
        model = Journal()
        if include_optional:
            return Journal(
                id = '',
                code = '',
                display_name = '',
                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                balancing_account_id = '',
                balancing_account_number = '',
                journal_lines = [
                    pybusinesscentral.model.journal_line.journalLine(
                        id = '', 
                        journal_display_name = '', 
                        line_number = 56, 
                        account_type = '', 
                        account_id = '', 
                        account_number = '', 
                        posting_date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        document_number = '', 
                        external_document_number = '', 
                        amount = 1.337, 
                        description = '', 
                        comment = '', 
                        dimensions = [
                            null
                            ], 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        attachments = [
                            pybusinesscentral.model.attachments.attachments(
                                parent_id = '', 
                                id = '', 
                                file_name = '', 
                                byte_size = 56, 
                                content = bytes(b'blah'), 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), )
                            ], 
                        account = pybusinesscentral.model.account.account(
                            id = '', 
                            number = '', 
                            display_name = '', 
                            category = '', 
                            sub_category = '', 
                            blocked = True, 
                            last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), )
                    ],
                account = pybusinesscentral.model.account.account(
                    id = '', 
                    number = '', 
                    display_name = '', 
                    category = '', 
                    sub_category = '', 
                    blocked = True, 
                    last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), )
            )
        else:
            return Journal(
        )
        """

    def testJournal(self):
        """Test Journal"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# flake8: noqa

# import all models into this package
# if you have many models here with many references from one model to another this may
# raise a RecursionError
# to avoid this, import only the models that you directly need like:
# from from pybusinesscentral.model.pet import Pet
# or import this package, but before doing it, use:
# import sys
# sys.setrecursionlimit(n)

from pybusinesscentral.model.account import Account
from pybusinesscentral.model.aged_accounts_payable import AgedAccountsPayable
from pybusinesscentral.model.aged_accounts_receivable import AgedAccountsReceivable
from pybusinesscentral.model.attachments import Attachments
from pybusinesscentral.model.balance_sheet import BalanceSheet
from pybusinesscentral.model.bank_account import BankAccount
from pybusinesscentral.model.cash_flow_statement import CashFlowStatement
from pybusinesscentral.model.company import Company
from pybusinesscentral.model.company_information import CompanyInformation
from pybusinesscentral.model.country_region import CountryRegion
from pybusinesscentral.model.currency import Currency
from pybusinesscentral.model.customer import Customer
from pybusinesscentral.model.customer_financial_detail import CustomerFinancialDetail
from pybusinesscentral.model.customer_payment import CustomerPayment
from pybusinesscentral.model.customer_payment_journal import CustomerPaymentJournal
from pybusinesscentral.model.customer_sale import CustomerSale
from pybusinesscentral.model.default_dimensions import DefaultDimensions
from pybusinesscentral.model.dimension import Dimension
from pybusinesscentral.model.dimension_line import DimensionLine
from pybusinesscentral.model.dimension_set_line import DimensionSetLine
from pybusinesscentral.model.dimension_value import DimensionValue
from pybusinesscentral.model.dimensiontype import Dimensiontype
from pybusinesscentral.model.documentlineobjectdetailstype import Documentlineobjectdetailstype
from pybusinesscentral.model.employee import Employee
from pybusinesscentral.model.general_ledger_entry import GeneralLedgerEntry
from pybusinesscentral.model.general_ledger_entry_attachments import GeneralLedgerEntryAttachments
from pybusinesscentral.model.income_statement import IncomeStatement
from pybusinesscentral.model.item import Item
from pybusinesscentral.model.item_category import ItemCategory
from pybusinesscentral.model.itemunitofmeasureconversiontype import Itemunitofmeasureconversiontype
from pybusinesscentral.model.journal import Journal
from pybusinesscentral.model.journal_line import JournalLine
from pybusinesscentral.model.payment_method import PaymentMethod
from pybusinesscentral.model.payment_term import PaymentTerm
from pybusinesscentral.model.pdf_document import PdfDocument
from pybusinesscentral.model.picture import Picture
from pybusinesscentral.model.postaladdresstype import Postaladdresstype
from pybusinesscentral.model.project import Project
from pybusinesscentral.model.purchase_invoice import PurchaseInvoice
from pybusinesscentral.model.purchase_invoice_line import PurchaseInvoiceLine
from pybusinesscentral.model.retained_earnings_statement import RetainedEarningsStatement
from pybusinesscentral.model.sales_credit_memo import SalesCreditMemo
from pybusinesscentral.model.sales_credit_memo_line import SalesCreditMemoLine
from pybusinesscentral.model.sales_invoice import SalesInvoice
from pybusinesscentral.model.sales_invoice_line import SalesInvoiceLine
from pybusinesscentral.model.sales_credit_memo import SalesCreditMemo
from pybusinesscentral.model.sales_credit_memo_line import SalesCreditMemoLine
from pybusinesscentral.model.sales_order import SalesOrder
from pybusinesscentral.model.sales_order_line import SalesOrderLine
from pybusinesscentral.model.sales_quote import SalesQuote
from pybusinesscentral.model.sales_quote_line import SalesQuoteLine
from pybusinesscentral.model.shipment_method import ShipmentMethod
from pybusinesscentral.model.tax_area import TaxArea
from pybusinesscentral.model.tax_group import TaxGroup
from pybusinesscentral.model.time_registration_entry import TimeRegistrationEntry
from pybusinesscentral.model.trial_balance import TrialBalance
from pybusinesscentral.model.unit_of_measure import UnitOfMeasure
from pybusinesscentral.model.unitofmeasuretype import Unitofmeasuretype
from pybusinesscentral.model.vendor import Vendor
from pybusinesscentral.model.vendor_purchase import VendorPurchase



# coding: utf-8

"""
    (v2.0) Dynamics 365 Business Central

    (v2.0) Business Central Standard APIs

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from pybusinesscentral.model.documentlineobjectdetailstype import Documentlineobjectdetailstype
from pybusinesscentral.model.unitofmeasuretype import Unitofmeasuretype
from typing import Optional, Set
from typing_extensions import Self

class PostSalesInvoiceLineForSalesInvoiceRequest(BaseModel):
    """
    PostSalesInvoiceLineForSalesInvoiceRequest
    """ # noqa: E501
    id: Optional[Annotated[str, Field(strict=True, max_length=50)]] = Field(default=None, description="(v1.0) The id property for the Dynamics 365 Business Central salesInvoiceLine entity")
    document_id: Optional[StrictStr] = Field(default=None, description="(v1.0) The documentId property for the Dynamics 365 Business Central salesInvoiceLine entity", alias="documentId")
    sequence: Optional[StrictInt] = Field(default=None, description="(v1.0) The sequence property for the Dynamics 365 Business Central salesInvoiceLine entity")
    item_id: Optional[StrictStr] = Field(default=None, description="(v1.0) The itemId property for the Dynamics 365 Business Central salesInvoiceLine entity", alias="itemId")
    account_id: Optional[StrictStr] = Field(default=None, description="(v1.0) The accountId property for the Dynamics 365 Business Central salesInvoiceLine entity", alias="accountId")
    line_type: Optional[StrictStr] = Field(default=None, description="(v1.0) The lineType property for the Dynamics 365 Business Central salesInvoiceLine entity", alias="lineType")
    line_details: Optional[Documentlineobjectdetailstype] = Field(default=None, alias="lineDetails")
    description: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="(v1.0) The description property for the Dynamics 365 Business Central salesInvoiceLine entity")
    unit_of_measure_id: Optional[StrictStr] = Field(default=None, description="(v1.0) The unitOfMeasureId property for the Dynamics 365 Business Central salesInvoiceLine entity", alias="unitOfMeasureId")
    unit_of_measure: Optional[Unitofmeasuretype] = Field(default=None, alias="unitOfMeasure")
    unit_price: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="(v1.0) The unitPrice property for the Dynamics 365 Business Central salesInvoiceLine entity", alias="unitPrice")
    quantity: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="(v1.0) The quantity property for the Dynamics 365 Business Central salesInvoiceLine entity")
    discount_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="(v1.0) The discountAmount property for the Dynamics 365 Business Central salesInvoiceLine entity", alias="discountAmount")
    discount_percent: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="(v1.0) The discountPercent property for the Dynamics 365 Business Central salesInvoiceLine entity", alias="discountPercent")
    discount_applied_before_tax: Optional[StrictBool] = Field(default=None, description="(v1.0) The discountAppliedBeforeTax property for the Dynamics 365 Business Central salesInvoiceLine entity", alias="discountAppliedBeforeTax")
    amount_excluding_tax: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="(v1.0) The amountExcludingTax property for the Dynamics 365 Business Central salesInvoiceLine entity", alias="amountExcludingTax")
    tax_code: Optional[Annotated[str, Field(strict=True, max_length=50)]] = Field(default=None, description="(v1.0) The taxCode property for the Dynamics 365 Business Central salesInvoiceLine entity", alias="taxCode")
    tax_percent: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="(v1.0) The taxPercent property for the Dynamics 365 Business Central salesInvoiceLine entity", alias="taxPercent")
    total_tax_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="(v1.0) The totalTaxAmount property for the Dynamics 365 Business Central salesInvoiceLine entity", alias="totalTaxAmount")
    amount_including_tax: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="(v1.0) The amountIncludingTax property for the Dynamics 365 Business Central salesInvoiceLine entity", alias="amountIncludingTax")
    invoice_discount_allocation: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="(v1.0) The invoiceDiscountAllocation property for the Dynamics 365 Business Central salesInvoiceLine entity", alias="invoiceDiscountAllocation")
    net_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="(v1.0) The netAmount property for the Dynamics 365 Business Central salesInvoiceLine entity", alias="netAmount")
    net_tax_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="(v1.0) The netTaxAmount property for the Dynamics 365 Business Central salesInvoiceLine entity", alias="netTaxAmount")
    net_amount_including_tax: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="(v1.0) The netAmountIncludingTax property for the Dynamics 365 Business Central salesInvoiceLine entity", alias="netAmountIncludingTax")
    shipment_date: Optional[date] = Field(default=None, description="(v1.0) The shipmentDate property for the Dynamics 365 Business Central salesInvoiceLine entity", alias="shipmentDate")
    __properties: ClassVar[List[str]] = ["id", "documentId", "sequence", "itemId", "accountId", "lineType", "lineDetails", "description", "unitOfMeasureId", "unitOfMeasure", "unitPrice", "quantity", "discountAmount", "discountPercent", "discountAppliedBeforeTax", "amountExcludingTax", "taxCode", "taxPercent", "totalTaxAmount", "amountIncludingTax", "invoiceDiscountAllocation", "netAmount", "netTaxAmount", "netAmountIncludingTax", "shipmentDate"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PostSalesInvoiceLineForSalesInvoiceRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of line_details
        if self.line_details:
            _dict['lineDetails'] = self.line_details.to_dict()
        # override the default output from pydantic by calling `to_dict()` of unit_of_measure
        if self.unit_of_measure:
            _dict['unitOfMeasure'] = self.unit_of_measure.to_dict()
        # set to None if document_id (nullable) is None
        # and model_fields_set contains the field
        if self.document_id is None and "document_id" in self.model_fields_set:
            _dict['documentId'] = None

        # set to None if sequence (nullable) is None
        # and model_fields_set contains the field
        if self.sequence is None and "sequence" in self.model_fields_set:
            _dict['sequence'] = None

        # set to None if item_id (nullable) is None
        # and model_fields_set contains the field
        if self.item_id is None and "item_id" in self.model_fields_set:
            _dict['itemId'] = None

        # set to None if account_id (nullable) is None
        # and model_fields_set contains the field
        if self.account_id is None and "account_id" in self.model_fields_set:
            _dict['accountId'] = None

        # set to None if line_type (nullable) is None
        # and model_fields_set contains the field
        if self.line_type is None and "line_type" in self.model_fields_set:
            _dict['lineType'] = None

        # set to None if line_details (nullable) is None
        # and model_fields_set contains the field
        if self.line_details is None and "line_details" in self.model_fields_set:
            _dict['lineDetails'] = None

        # set to None if description (nullable) is None
        # and model_fields_set contains the field
        if self.description is None and "description" in self.model_fields_set:
            _dict['description'] = None

        # set to None if unit_of_measure_id (nullable) is None
        # and model_fields_set contains the field
        if self.unit_of_measure_id is None and "unit_of_measure_id" in self.model_fields_set:
            _dict['unitOfMeasureId'] = None

        # set to None if unit_of_measure (nullable) is None
        # and model_fields_set contains the field
        if self.unit_of_measure is None and "unit_of_measure" in self.model_fields_set:
            _dict['unitOfMeasure'] = None

        # set to None if unit_price (nullable) is None
        # and model_fields_set contains the field
        if self.unit_price is None and "unit_price" in self.model_fields_set:
            _dict['unitPrice'] = None

        # set to None if quantity (nullable) is None
        # and model_fields_set contains the field
        if self.quantity is None and "quantity" in self.model_fields_set:
            _dict['quantity'] = None

        # set to None if discount_amount (nullable) is None
        # and model_fields_set contains the field
        if self.discount_amount is None and "discount_amount" in self.model_fields_set:
            _dict['discountAmount'] = None

        # set to None if discount_percent (nullable) is None
        # and model_fields_set contains the field
        if self.discount_percent is None and "discount_percent" in self.model_fields_set:
            _dict['discountPercent'] = None

        # set to None if discount_applied_before_tax (nullable) is None
        # and model_fields_set contains the field
        if self.discount_applied_before_tax is None and "discount_applied_before_tax" in self.model_fields_set:
            _dict['discountAppliedBeforeTax'] = None

        # set to None if amount_excluding_tax (nullable) is None
        # and model_fields_set contains the field
        if self.amount_excluding_tax is None and "amount_excluding_tax" in self.model_fields_set:
            _dict['amountExcludingTax'] = None

        # set to None if tax_code (nullable) is None
        # and model_fields_set contains the field
        if self.tax_code is None and "tax_code" in self.model_fields_set:
            _dict['taxCode'] = None

        # set to None if tax_percent (nullable) is None
        # and model_fields_set contains the field
        if self.tax_percent is None and "tax_percent" in self.model_fields_set:
            _dict['taxPercent'] = None

        # set to None if total_tax_amount (nullable) is None
        # and model_fields_set contains the field
        if self.total_tax_amount is None and "total_tax_amount" in self.model_fields_set:
            _dict['totalTaxAmount'] = None

        # set to None if amount_including_tax (nullable) is None
        # and model_fields_set contains the field
        if self.amount_including_tax is None and "amount_including_tax" in self.model_fields_set:
            _dict['amountIncludingTax'] = None

        # set to None if invoice_discount_allocation (nullable) is None
        # and model_fields_set contains the field
        if self.invoice_discount_allocation is None and "invoice_discount_allocation" in self.model_fields_set:
            _dict['invoiceDiscountAllocation'] = None

        # set to None if net_amount (nullable) is None
        # and model_fields_set contains the field
        if self.net_amount is None and "net_amount" in self.model_fields_set:
            _dict['netAmount'] = None

        # set to None if net_tax_amount (nullable) is None
        # and model_fields_set contains the field
        if self.net_tax_amount is None and "net_tax_amount" in self.model_fields_set:
            _dict['netTaxAmount'] = None

        # set to None if net_amount_including_tax (nullable) is None
        # and model_fields_set contains the field
        if self.net_amount_including_tax is None and "net_amount_including_tax" in self.model_fields_set:
            _dict['netAmountIncludingTax'] = None

        # set to None if shipment_date (nullable) is None
        # and model_fields_set contains the field
        if self.shipment_date is None and "shipment_date" in self.model_fields_set:
            _dict['shipmentDate'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PostSalesInvoiceLineForSalesInvoiceRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "documentId": obj.get("documentId"),
            "sequence": obj.get("sequence"),
            "itemId": obj.get("itemId"),
            "accountId": obj.get("accountId"),
            "lineType": obj.get("lineType"),
            "lineDetails": Documentlineobjectdetailstype.from_dict(obj["lineDetails"]) if obj.get("lineDetails") is not None else None,
            "description": obj.get("description"),
            "unitOfMeasureId": obj.get("unitOfMeasureId"),
            "unitOfMeasure": Unitofmeasuretype.from_dict(obj["unitOfMeasure"]) if obj.get("unitOfMeasure") is not None else None,
            "unitPrice": obj.get("unitPrice"),
            "quantity": obj.get("quantity"),
            "discountAmount": obj.get("discountAmount"),
            "discountPercent": obj.get("discountPercent"),
            "discountAppliedBeforeTax": obj.get("discountAppliedBeforeTax"),
            "amountExcludingTax": obj.get("amountExcludingTax"),
            "taxCode": obj.get("taxCode"),
            "taxPercent": obj.get("taxPercent"),
            "totalTaxAmount": obj.get("totalTaxAmount"),
            "amountIncludingTax": obj.get("amountIncludingTax"),
            "invoiceDiscountAllocation": obj.get("invoiceDiscountAllocation"),
            "netAmount": obj.get("netAmount"),
            "netTaxAmount": obj.get("netTaxAmount"),
            "netAmountIncludingTax": obj.get("netAmountIncludingTax"),
            "shipmentDate": obj.get("shipmentDate")
        })
        return _obj





# coding: utf-8

"""
    (v2.0) Dynamics 365 Business Central

    (v2.0) Business Central Standard APIs

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from pybusinesscentral.model.default_dimensions import DefaultDimensions
from pybusinesscentral.model.picture import Picture
from pybusinesscentral.model.postaladdresstype import Postaladdresstype
from pybusinesscentral.model.time_registration_entry import TimeRegistrationEntry
from typing import Optional, Set
from typing_extensions import Self

class Employee(BaseModel):
    """
    Employee
    """ # noqa: E501
    id: Optional[StrictStr] = Field(default=None, description="(v1.0) The id property for the Dynamics 365 Business Central employee entity")
    number: Optional[Annotated[str, Field(strict=True, max_length=20)]] = Field(default=None, description="(v1.0) The number property for the Dynamics 365 Business Central employee entity")
    display_name: Optional[StrictStr] = Field(default=None, description="(v1.0) The displayName property for the Dynamics 365 Business Central employee entity", alias="displayName")
    given_name: Optional[Annotated[str, Field(strict=True, max_length=30)]] = Field(default=None, description="(v1.0) The givenName property for the Dynamics 365 Business Central employee entity", alias="givenName")
    middle_name: Optional[Annotated[str, Field(strict=True, max_length=30)]] = Field(default=None, description="(v1.0) The middleName property for the Dynamics 365 Business Central employee entity", alias="middleName")
    surname: Optional[Annotated[str, Field(strict=True, max_length=30)]] = Field(default=None, description="(v1.0) The surname property for the Dynamics 365 Business Central employee entity")
    job_title: Optional[Annotated[str, Field(strict=True, max_length=30)]] = Field(default=None, description="(v1.0) The jobTitle property for the Dynamics 365 Business Central employee entity", alias="jobTitle")
    address: Optional[Postaladdresstype] = None
    phone_number: Optional[Annotated[str, Field(strict=True, max_length=30)]] = Field(default=None, description="(v1.0) The phoneNumber property for the Dynamics 365 Business Central employee entity", alias="phoneNumber")
    mobile_phone: Optional[Annotated[str, Field(strict=True, max_length=30)]] = Field(default=None, description="(v1.0) The mobilePhone property for the Dynamics 365 Business Central employee entity", alias="mobilePhone")
    email: Optional[Annotated[str, Field(strict=True, max_length=80)]] = Field(default=None, description="(v1.0) The email property for the Dynamics 365 Business Central employee entity")
    personal_email: Optional[Annotated[str, Field(strict=True, max_length=80)]] = Field(default=None, description="(v1.0) The personalEmail property for the Dynamics 365 Business Central employee entity", alias="personalEmail")
    employment_date: Optional[datetime] = Field(default=None, description="(v1.0) The employmentDate property for the Dynamics 365 Business Central employee entity", alias="employmentDate")
    termination_date: Optional[datetime] = Field(default=None, description="(v1.0) The terminationDate property for the Dynamics 365 Business Central employee entity", alias="terminationDate")
    status: Optional[StrictStr] = Field(default=None, description="(v1.0) The status property for the Dynamics 365 Business Central employee entity")
    birth_date: Optional[datetime] = Field(default=None, description="(v1.0) The birthDate property for the Dynamics 365 Business Central employee entity", alias="birthDate")
    statistics_group_code: Optional[Annotated[str, Field(strict=True, max_length=10)]] = Field(default=None, description="(v1.0) The statisticsGroupCode property for the Dynamics 365 Business Central employee entity", alias="statisticsGroupCode")
    last_modified_date_time: Optional[datetime] = Field(default=None, description="(v1.0) The lastModifiedDateTime property for the Dynamics 365 Business Central employee entity", alias="lastModifiedDateTime")
    picture: Optional[List[Picture]] = None
    default_dimensions: Optional[List[DefaultDimensions]] = Field(default=None, alias="defaultDimensions")
    time_registration_entries: Optional[List[TimeRegistrationEntry]] = Field(default=None, alias="timeRegistrationEntries")
    __properties: ClassVar[List[str]] = ["id", "number", "displayName", "givenName", "middleName", "surname", "jobTitle", "address", "phoneNumber", "mobilePhone", "email", "personalEmail", "employmentDate", "terminationDate", "status", "birthDate", "statisticsGroupCode", "lastModifiedDateTime", "picture", "defaultDimensions", "timeRegistrationEntries"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Employee from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of address
        if self.address:
            _dict['address'] = self.address.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in picture (list)
        _items = []
        if self.picture:
            for _item_picture in self.picture:
                if _item_picture:
                    _items.append(_item_picture.to_dict())
            _dict['picture'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in default_dimensions (list)
        _items = []
        if self.default_dimensions:
            for _item_default_dimensions in self.default_dimensions:
                if _item_default_dimensions:
                    _items.append(_item_default_dimensions.to_dict())
            _dict['defaultDimensions'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in time_registration_entries (list)
        _items = []
        if self.time_registration_entries:
            for _item_time_registration_entries in self.time_registration_entries:
                if _item_time_registration_entries:
                    _items.append(_item_time_registration_entries.to_dict())
            _dict['timeRegistrationEntries'] = _items
        # set to None if number (nullable) is None
        # and model_fields_set contains the field
        if self.number is None and "number" in self.model_fields_set:
            _dict['number'] = None

        # set to None if display_name (nullable) is None
        # and model_fields_set contains the field
        if self.display_name is None and "display_name" in self.model_fields_set:
            _dict['displayName'] = None

        # set to None if given_name (nullable) is None
        # and model_fields_set contains the field
        if self.given_name is None and "given_name" in self.model_fields_set:
            _dict['givenName'] = None

        # set to None if middle_name (nullable) is None
        # and model_fields_set contains the field
        if self.middle_name is None and "middle_name" in self.model_fields_set:
            _dict['middleName'] = None

        # set to None if surname (nullable) is None
        # and model_fields_set contains the field
        if self.surname is None and "surname" in self.model_fields_set:
            _dict['surname'] = None

        # set to None if job_title (nullable) is None
        # and model_fields_set contains the field
        if self.job_title is None and "job_title" in self.model_fields_set:
            _dict['jobTitle'] = None

        # set to None if address (nullable) is None
        # and model_fields_set contains the field
        if self.address is None and "address" in self.model_fields_set:
            _dict['address'] = None

        # set to None if phone_number (nullable) is None
        # and model_fields_set contains the field
        if self.phone_number is None and "phone_number" in self.model_fields_set:
            _dict['phoneNumber'] = None

        # set to None if mobile_phone (nullable) is None
        # and model_fields_set contains the field
        if self.mobile_phone is None and "mobile_phone" in self.model_fields_set:
            _dict['mobilePhone'] = None

        # set to None if email (nullable) is None
        # and model_fields_set contains the field
        if self.email is None and "email" in self.model_fields_set:
            _dict['email'] = None

        # set to None if personal_email (nullable) is None
        # and model_fields_set contains the field
        if self.personal_email is None and "personal_email" in self.model_fields_set:
            _dict['personalEmail'] = None

        # set to None if employment_date (nullable) is None
        # and model_fields_set contains the field
        if self.employment_date is None and "employment_date" in self.model_fields_set:
            _dict['employmentDate'] = None

        # set to None if termination_date (nullable) is None
        # and model_fields_set contains the field
        if self.termination_date is None and "termination_date" in self.model_fields_set:
            _dict['terminationDate'] = None

        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict['status'] = None

        # set to None if birth_date (nullable) is None
        # and model_fields_set contains the field
        if self.birth_date is None and "birth_date" in self.model_fields_set:
            _dict['birthDate'] = None

        # set to None if statistics_group_code (nullable) is None
        # and model_fields_set contains the field
        if self.statistics_group_code is None and "statistics_group_code" in self.model_fields_set:
            _dict['statisticsGroupCode'] = None

        # set to None if last_modified_date_time (nullable) is None
        # and model_fields_set contains the field
        if self.last_modified_date_time is None and "last_modified_date_time" in self.model_fields_set:
            _dict['lastModifiedDateTime'] = None

        # set to None if picture (nullable) is None
        # and model_fields_set contains the field
        if self.picture is None and "picture" in self.model_fields_set:
            _dict['picture'] = None

        # set to None if default_dimensions (nullable) is None
        # and model_fields_set contains the field
        if self.default_dimensions is None and "default_dimensions" in self.model_fields_set:
            _dict['defaultDimensions'] = None

        # set to None if time_registration_entries (nullable) is None
        # and model_fields_set contains the field
        if self.time_registration_entries is None and "time_registration_entries" in self.model_fields_set:
            _dict['timeRegistrationEntries'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Employee from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "number": obj.get("number"),
            "displayName": obj.get("displayName"),
            "givenName": obj.get("givenName"),
            "middleName": obj.get("middleName"),
            "surname": obj.get("surname"),
            "jobTitle": obj.get("jobTitle"),
            "address": Postaladdresstype.from_dict(obj["address"]) if obj.get("address") is not None else None,
            "phoneNumber": obj.get("phoneNumber"),
            "mobilePhone": obj.get("mobilePhone"),
            "email": obj.get("email"),
            "personalEmail": obj.get("personalEmail"),
            "employmentDate": obj.get("employmentDate"),
            "terminationDate": obj.get("terminationDate"),
            "status": obj.get("status"),
            "birthDate": obj.get("birthDate"),
            "statisticsGroupCode": obj.get("statisticsGroupCode"),
            "lastModifiedDateTime": obj.get("lastModifiedDateTime"),
            "picture": [Picture.from_dict(_item) for _item in obj["picture"]] if obj.get("picture") is not None else None,
            "defaultDimensions": [DefaultDimensions.from_dict(_item) for _item in obj["defaultDimensions"]] if obj.get("defaultDimensions") is not None else None,
            "timeRegistrationEntries": [TimeRegistrationEntry.from_dict(_item) for _item in obj["timeRegistrationEntries"]] if obj.get("timeRegistrationEntries") is not None else None
        })
        return _obj


